@@ -47,7 +47,7 @@
  * This is a reference counted opaque structure type.  It is not a
  * #GObject.
  *
- * Use dconf_changeset_ref() and dconf_changeset_unref() to manipuate
+ * Use dconf_changeset_ref() and dconf_changeset_unref() to manipulate
  * references.
  **/
 
@@ -90,7 +90,7 @@ dconf_changeset_new (void)
 
 /**
  * dconf_changeset_new_database:
- * @copy_of: (allow none): a #DConfChangeset to copy
+ * @copy_of: (allow-none): a #DConfChangeset to copy
  *
  * Creates a new #DConfChangeset in "database" mode, possibly
  * initialising it with the values of another changeset.
@@ -334,7 +334,7 @@ dconf_changeset_is_similar_to (DConfChangeset *changeset,
  * @predicate is called on each item in the changeset, in turn, until it
  * returns %FALSE.
  *
- * If @preciate returns %FALSE for any item, this function returns
+ * If @predicate returns %FALSE for any item, this function returns
  * %FALSE.  If not (including the case of no items) then this function
  * returns %TRUE.
  *
@@ -387,8 +387,9 @@ dconf_changeset_build_description (DConfChangeset *changeset)
    * Next, we iterate the table again to pull the strings out excluding
    * the leading prefix.
    *
-   * We sort the list of paths at this point because the rebuilder
-   * requires a sorted list.
+   * We sort the list of paths at this point because the writer
+   * requires a sorted list in order to ensure that dir resets come
+   * before writes to keys in that dir.
    *
    * Finally, we iterate over the sorted list and use the normal
    * hashtable lookup in order to populate the values array in the same
@@ -562,7 +563,7 @@ dconf_changeset_serialise (DConfChangeset *changeset)
  * returned from an earlier call to dconf_changeset_serialise().
  *
  * @serialised has no particular format -- you should only pass a value
- * that reasulted from an earlier serialise operation.
+ * that resulted from an earlier serialise operation.
  *
  * This call never fails, even if @serialised is not in the correct
  * format.  Improperly-formatted parts are simply ignored.
@@ -697,3 +698,76 @@ dconf_changeset_change (DConfChangeset *changeset,
       dconf_changeset_set (changeset, path, value);
     }
 }
+
+/**
+ * dconf_changeset_diff:
+ * @from: a database mode changeset
+ * @to: a database mode changeset
+ *
+ * Compares to database-mode changesets and produces a changeset that
+ * describes their differences.
+ *
+ * If there is no difference, %NULL is returned.
+ *
+ * Applying the returned changeset to @from using
+ * dconf_changeset_change() will result in the two changesets being
+ * equal.
+ *
+ * Returns: (transfer full): the changes, or %NULL
+ *
+ * Since: 0.16
+ */
+DConfChangeset *
+dconf_changeset_diff (DConfChangeset *from,
+                      DConfChangeset *to)
+{
+  DConfChangeset *changeset = NULL;
+  GHashTableIter iter;
+  gpointer key, val;
+
+  g_return_val_if_fail (from->is_database, NULL);
+  g_return_val_if_fail (to->is_database, NULL);
+
+  /* We make no attempt to do dir resets, but we could...
+   *
+   * For now, we just reset each key individually.
+   *
+   * We create our list of changes in two steps:
+   *
+   *   - iterate the 'to' changeset and note any keys that do not have
+   *     the same value in the 'from' changeset
+   *
+   *   - iterate the 'from' changeset and note any keys not present in
+   *     the 'to' changeset, recording resets for them
+   *
+   * This will cover all changes.
+   *
+   * Note: because 'from' and 'to' are database changesets we don't have
+   * to worry about seeing NULL values or dirs.
+   */
+  g_hash_table_iter_init (&iter, to->table);
+  while (g_hash_table_iter_next (&iter, &key, &val))
+    {
+      GVariant *from_val = g_hash_table_lookup (from->table, key);
+
+      if (from_val == NULL || !g_variant_equal (val, from_val))
+        {
+          if (!changeset)
+            changeset = dconf_changeset_new ();
+
+          dconf_changeset_set (changeset, key, val);
+        }
+    }
+
+  g_hash_table_iter_init (&iter, from->table);
+  while (g_hash_table_iter_next (&iter, &key, &val))
+    if (!g_hash_table_lookup (to->table, key))
+      {
+        if (!changeset)
+          changeset = dconf_changeset_new ();
+
+        dconf_changeset_set (changeset, key, NULL);
+      }
+
+  return changeset;
+}
