@@ -25,6 +25,7 @@
 #include "../shm/dconf-shm.h"
 #include "dconf-gvdb-utils.h"
 #include "dconf-generated.h"
+#include "dconf-blame.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -33,20 +34,21 @@
 #include <errno.h>
 #include <stdio.h>
 
-typedef struct
+struct _DConfWriterPrivate
 {
-  DConfDBusWriterSkeleton parent_instance;
   gchar *filename;
   gboolean native;
+  gchar *basepath;
   gchar *name;
   guint64 tag;
+  gboolean need_write;
 
   DConfChangeset *uncommited_values;
   DConfChangeset *commited_values;
 
   GQueue uncommited_changes;
   GQueue commited_changes;
-} DConfWriter;
+};
 
 typedef struct
 {
@@ -54,26 +56,33 @@ typedef struct
   gchar          *tag;
 } TaggedChange;
 
-typedef struct
-{
-  DConfDBusWriterSkeletonClass parent_instance;
-
-  gboolean (* begin)  (DConfWriter     *writer,
-                       GError         **error);
-  void     (* change) (DConfWriter     *writer,
-                       DConfChangeset  *changeset,
-                       const gchar     *tag);
-  gboolean (* commit) (DConfWriter     *writer,
-                       GError         **error);
-  void     (* end)    (DConfWriter     *writer);
-} DConfWriterClass;
-
-
 static void dconf_writer_iface_init (DConfDBusWriterIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (DConfWriter, dconf_writer, DCONF_DBUS_TYPE_WRITER_SKELETON,
                          G_IMPLEMENT_INTERFACE (DCONF_DBUS_TYPE_WRITER, dconf_writer_iface_init))
 
+static void
+dconf_writer_real_list (GHashTable *set)
+{
+  const gchar *name;
+  gchar *dirname;
+  GDir *dir;
+
+  dirname = g_build_filename (g_get_user_config_dir (), "dconf", NULL);
+  dir = g_dir_open (dirname, 0, NULL);
+
+  if (!dir)
+    return;
+
+  while ((name = g_dir_read_name (dir)))
+    {
+      if (!strchr (name, '.'))
+        g_hash_table_add (set, g_strdup (name));
+    }
+
+  g_dir_close (dir);
+}
+
 static gchar *
 dconf_writer_get_tag (DConfWriter *writer)
 {
@@ -83,7 +92,7 @@ dconf_writer_get_tag (DConfWriter *writer)
 
   return g_strdup_printf ("%s:%s:%" G_GUINT64_FORMAT,
                           g_dbus_connection_get_unique_name (connection),
-                          writer->name, writer->tag++);
+                          writer->priv->name, writer->priv->tag++);
 }
 
 static gboolean
@@ -93,15 +102,23 @@ dconf_writer_real_begin (DConfWriter  *writer,
   /* If this is the first time, populate the value table with the
    * existing values.
    */
-  if (writer->commited_values == NULL)
+  if (writer->priv->commited_values == NULL)
     {
-      writer->commited_values = dconf_gvdb_utils_read_file (writer->filename, error);
+      gboolean missing;
 
-      if (!writer->commited_values)
+      writer->priv->commited_values = dconf_gvdb_utils_read_file (writer->priv->filename, &missing, error);
+
+      if (!writer->priv->commited_values)
         return FALSE;
+
+      /* If this is a non-native writer and the file doesn't exist, we
+       * will need to write it on commit so that the client can open it.
+       */
+      if (missing && !writer->priv->native)
+        writer->priv->need_write = TRUE;
     }
 
-  writer->uncommited_values = dconf_changeset_new_database (writer->commited_values);
+  writer->priv->uncommited_values = dconf_changeset_new_database (writer->priv->commited_values);
 
   return TRUE;
 }
@@ -111,9 +128,9 @@ dconf_writer_real_change (DConfWriter    *writer,
                           DConfChangeset *changeset,
                           const gchar    *tag)
 {
-  g_return_if_fail (writer->uncommited_values != NULL);
+  g_return_if_fail (writer->priv->uncommited_values != NULL);
 
-  dconf_changeset_change (writer->uncommited_values, changeset);
+  dconf_changeset_change (writer->priv->uncommited_values, changeset);
 
   if (tag)
     {
@@ -123,31 +140,55 @@ dconf_writer_real_change (DConfWriter    *writer,
       change->changeset = dconf_changeset_ref (changeset);
       change->tag = g_strdup (tag);
 
-      g_queue_push_tail (&writer->uncommited_changes, change);
+      g_queue_push_tail (&writer->priv->uncommited_changes, change);
     }
+
+  writer->priv->need_write = TRUE;
 }
 
 static gboolean
 dconf_writer_real_commit (DConfWriter  *writer,
                           GError      **error)
 {
-  if (!dconf_gvdb_utils_write_file (writer->filename, writer->uncommited_values, error))
+  gint invalidate_fd = -1;
+
+  if (!writer->priv->need_write)
+    {
+      g_assert (g_queue_is_empty (&writer->priv->uncommited_changes));
+      g_assert (g_queue_is_empty (&writer->priv->commited_changes));
+      dconf_changeset_unref (writer->priv->uncommited_values);
+      writer->priv->uncommited_values = NULL;
+
+      return TRUE;
+    }
+
+  if (!writer->priv->native)
+    /* If it fails, it doesn't matter... */
+    invalidate_fd = open (writer->priv->filename, O_WRONLY);
+
+  if (!dconf_gvdb_utils_write_file (writer->priv->filename, writer->priv->uncommited_values, error))
     return FALSE;
 
-  if (writer->native)
-    dconf_shm_flag (writer->name);
+  if (writer->priv->native)
+    dconf_shm_flag (writer->priv->name);
 
-  if (writer->commited_values)
-    dconf_changeset_unref (writer->commited_values);
-  writer->commited_values = writer->uncommited_values;
-  writer->uncommited_values = NULL;
+  if (invalidate_fd != -1)
+    {
+      write (invalidate_fd, "\0\0\0\0\0\0\0\0", 8);
+      close (invalidate_fd);
+    }
+
+  if (writer->priv->commited_values)
+    dconf_changeset_unref (writer->priv->commited_values);
+  writer->priv->commited_values = writer->priv->uncommited_values;
+  writer->priv->uncommited_values = NULL;
 
   {
     GQueue empty_queue = G_QUEUE_INIT;
 
-    g_assert (g_queue_is_empty (&writer->commited_changes));
-    writer->commited_changes = writer->uncommited_changes;
-    writer->uncommited_changes = empty_queue;
+    g_assert (g_queue_is_empty (&writer->priv->commited_changes));
+    writer->priv->commited_changes = writer->priv->uncommited_changes;
+    writer->priv->uncommited_changes = empty_queue;
   }
 
   return TRUE;
@@ -156,27 +197,30 @@ dconf_writer_real_commit (DConfWriter  *writer,
 static void
 dconf_writer_real_end (DConfWriter *writer)
 {
-  while (!g_queue_is_empty (&writer->uncommited_changes))
+  while (!g_queue_is_empty (&writer->priv->uncommited_changes))
     {
-      TaggedChange *change = g_queue_pop_head (&writer->commited_changes);
+      TaggedChange *change = g_queue_pop_head (&writer->priv->uncommited_changes);
+      dconf_changeset_unref (change->changeset);
       g_free (change->tag);
       g_slice_free (TaggedChange, change);
     }
 
-  while (!g_queue_is_empty (&writer->commited_changes))
+  while (!g_queue_is_empty (&writer->priv->commited_changes))
     {
-      TaggedChange *change = g_queue_pop_head (&writer->commited_changes);
+      TaggedChange *change = g_queue_pop_head (&writer->priv->commited_changes);
       const gchar *prefix;
       const gchar * const *paths;
+      guint n;
 
-      dconf_changeset_describe (change->changeset, &prefix, &paths, NULL);
+      n = dconf_changeset_describe (change->changeset, &prefix, &paths, NULL);
+      g_assert (n != 0);
       dconf_dbus_writer_emit_notify_signal (DCONF_DBUS_WRITER (writer), prefix, paths, change->tag);
       dconf_changeset_unref (change->changeset);
       g_free (change->tag);
       g_slice_free (TaggedChange, change);
     }
 
-  g_clear_pointer (&writer->uncommited_values, g_hash_table_unref);
+  g_clear_pointer (&writer->priv->uncommited_values, dconf_changeset_unref);
 }
 
 gboolean
@@ -257,16 +301,21 @@ dconf_writer_handle_change (DConfDBusWriter       *dbus_writer,
 
   tag = dconf_writer_get_tag (writer);
 
-  if (!dconf_writer_begin (writer, &error))
-    goto out;
+  /* Don't bother with empty changesets... */
+  if (dconf_changeset_describe (changeset, NULL, NULL, NULL))
+    {
+      if (!dconf_writer_begin (writer, &error))
+        goto out;
 
-  dconf_writer_change (writer, changeset, tag);
-  dconf_changeset_unref (changeset);
+      dconf_writer_change (writer, changeset, tag);
 
-  if (!dconf_writer_commit (writer, &error))
-    goto out;
+      if (!dconf_writer_commit (writer, &error))
+        goto out;
+    }
 
 out:
+  dconf_changeset_unref (changeset);
+
   if (error)
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -293,7 +342,9 @@ dconf_writer_iface_init (DConfDBusWriterIface *iface)
 static void
 dconf_writer_init (DConfWriter *writer)
 {
-  writer->native = TRUE;
+  writer->priv = G_TYPE_INSTANCE_GET_PRIVATE (writer, DCONF_TYPE_WRITER, DConfWriterPrivate);
+  writer->priv->basepath = g_build_filename (g_get_user_config_dir (), "dconf", NULL);
+  writer->priv->native = TRUE;
 }
 
 static void
@@ -304,13 +355,10 @@ dconf_writer_set_property (GObject *object, guint prop_id,
 
   g_assert_cmpint (prop_id, ==, 1);
 
-  g_assert (!writer->name);
-  writer->name = g_value_dup_string (value);
+  g_assert (!writer->priv->name);
+  writer->priv->name = g_value_dup_string (value);
 
-  if (writer->native)
-    writer->filename = g_build_filename (g_get_user_config_dir (), "dconf", writer->name, NULL);
-  else
-    writer->filename = g_build_filename (g_get_user_runtime_dir (), "dconf", writer->name, NULL);
+  writer->priv->filename = g_build_filename (writer->priv->basepath, writer->priv->name, NULL);
 }
 
 static void
@@ -324,21 +372,56 @@ dconf_writer_class_init (DConfWriterClass *class)
   class->change = dconf_writer_real_change;
   class->commit = dconf_writer_real_commit;
   class->end = dconf_writer_real_end;
+  class->list = dconf_writer_real_list;
 
   g_object_class_install_property (object_class, 1,
                                    g_param_spec_string ("name", "name", "name", NULL,
                                                         G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_WRITABLE));
+
+  g_type_class_add_private (class, sizeof (DConfWriterPrivate));
+}
+
+void
+dconf_writer_set_basepath (DConfWriter *writer,
+                           const gchar *name)
+{
+  g_free (writer->priv->basepath);
+  writer->priv->basepath = g_build_filename (g_get_user_runtime_dir (), "dconf-service", name, NULL);
+  writer->priv->native = FALSE;
+}
+
+DConfChangeset *
+dconf_writer_diff (DConfWriter    *writer,
+                   DConfChangeset *changeset)
+{
+  return dconf_changeset_diff (writer->priv->uncommited_values, changeset);
 }
 
 const gchar *
 dconf_writer_get_name (DConfWriter *writer)
 {
-  return writer->name;
+  return writer->priv->name;
+}
+
+void
+dconf_writer_list (GType       type,
+                   GHashTable *set)
+{
+  DConfWriterClass *class;
+
+  g_return_if_fail (g_type_is_a (type, DCONF_TYPE_WRITER));
+
+  class = g_type_class_ref (type);
+  class->list (set);
+  g_type_class_unref (class);
 }
 
 GDBusInterfaceSkeleton *
-dconf_writer_new (const gchar *name)
+dconf_writer_new (GType        type,
+                  const gchar *name)
 {
-  return g_object_new (DCONF_TYPE_WRITER, "name", name, NULL);
+  g_return_val_if_fail (g_type_is_a (type, DCONF_TYPE_WRITER), NULL);
+
+  return g_object_new (type, "name", name, NULL);
 }
